/*
 * process_list.hpp
 * Copyright (C) Tobias Groenhagen 2009
 *
 */

#include "proc.hpp"
#include <fstream>
#include <sstream>
#include <algorithm>
#include <iomanip>
#include <QtGui>

using namespace std;

Process::Process(string proc, int pid)
: path(proc), proc(proc), pid(pid)
{
  if(pid > 0)
  {
    ostringstream sstr;
    sstr << "/" << pid;
    path+=sstr.str();
  }
  init();
}

void Process::init()
{
  root=false;
  if(pid>-1)
  {
    if(pid==0)
    {
      pid  =0;
      name ="Kernel";
      state = ' ';
      ppid =-1;
      pgrp =-1;
      session=0;
      tty_nr =-1;
      tty_grp =-1;
      nice =-20;
      num_threads=1;
      read_kernel_stat();
    }
    else
      read_stat();
    read_cmd();
  }
  else
  {
    cmd="DUMMY";
    root=true;
    pid  =-1;
    name ="ROOT";
    state = 'O';
    ppid =-2;
  }
}

void Process::read_cmd()
{
  fstream file((path+"/cmdline").c_str(),ios::in);
  getline(file,cmd);
  file.close();
  unsigned int pos=cmd.find('\0');
  while(pos!=string::npos)
  {
  	cmd.replace(pos,1,1,' ');
    pos=cmd.find('\0');
  }
}

void Process::read_stat()
{
  fstream file((path+"/stat").c_str(),ios::in);
  file >> pid >> name >> state >> ppid >> pgrp >> session;
  file >> tty_nr >> tty_grp >> flags >> min_flt;
  file >> cmin_flt >> maj_flt >> cmaj_flt >> utime >> stime;
  file >> cutime >> cstime >> priority >> nice >> num_threads;
  file >> unused1 >> start_time >> vsize >> rss >> rsslim;
  file >> start_code >> end_code >> start_stack >> esp >> eip;
  file >> pending >> blocked >> sigign >> sigcatch >> wchan;
  file >> unused2 >> unused3 >> exit_signal >> task_cpu;
  file >>  rt_priority >> policy;
  file.close();

  if(name.size() > 1)
  { name=name.substr(1,name.size()-2); }
}

void Process::read_kernel_stat()
{
  fstream file((proc+"/uptime").c_str(),ios::in);
  file >> start_time >> cstime;
  file.close();
  stime = cstime;
}

int           Process::getPID() const { return pid; }
int           Process::getPPID() const { return ppid; }
std::string   Process::getName() const { return name; }
std::string   Process::getCMD() const { return cmd; }
char          Process::getState() const { return state; }
int           Process::getTTY() const { return tty_nr; }
unsigned long Process::getUTime() const { return utime; }
unsigned long Process::getSTime() const { return stime; }
unsigned long Process::getCUTime() const { return cutime; }
unsigned long Process::getCSTime() const { return cstime; }
long          Process::getNice() const { return nice; }
unsigned long Process::getStartTime() const { return start_time; }
bool          Process::isRoot() const { return root; }

std::string Process::getLine() const
{
  ostringstream sstr;
  sstr << right << setw(10) << pid << " : ";
  sstr << "(" << left << setw(15) << name << ") ";
  sstr << state << " ";
  sstr << "\"" << cmd << "\"";
  return sstr.str();
}

bool Process::operator< (const Process& p) const
{
  return pid < p.getPID();
}

//######################################################################

ProcessList::ProcessList(std::string p)
: path(p)
{
  list = new std::vector<Process>();
  readProcesses();
}
ProcessList::~ProcessList()
{ delete(list); }

//--------------------------------------------------------------------
std::vector <int> ProcessList::readPIDs()
{
  std::vector <int> plist;
  plist.push_back(-1);
  plist.push_back(0);
  QString p(path.c_str());
  QDir proc(p);
  proc.setFilter(QDir::Dirs);
  proc.setSorting(QDir::Name);
  if(proc.exists())
  {
    QListIterator <QFileInfo> iter(proc.entryInfoList());
    while(iter.hasNext())
    {
      stringstream sstr(iter.next().fileName().toStdString());
      int pid =0;
      sstr >> pid;
      if(pid>0) plist.push_back(pid);
    }
  }
  std::sort(plist.begin(),plist.end());
  return plist;
}

void ProcessList::readProcesses()
{
  std::vector <int> pids=readPIDs();
  for(unsigned int i=0; i< pids.size(); i++)
  {
    list->push_back(Process(path, pids[i]));
  }
}
//--------------------------------------------------------------------

std::vector <int> ProcessList::getPIDs()
{
  std::vector <int> pids;
  for(unsigned int i=0; i< list->size(); i++)
    pids.push_back(list->at(i).getPID());
  return pids;
}

Process* ProcessList::getProcess(int pid)
{
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPID() == pid) return &list->at(i);
  }
  return getProcess(-1);
}

std::vector<Process>* ProcessList::getProcesses()
{ return list; }

std::vector<int> ProcessList::getChildPIDs(int pid)
{
  vector<int> cplist;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == pid) cplist.push_back(list->at(i).getPID());
  }
  return cplist;
}

std::vector<Process*> ProcessList::getChildProcesses(int pid)
{
  vector<Process*> cplist;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == pid) cplist.push_back(&list->at(i));
  }
  return cplist;
}

unsigned int ProcessList::getChildPosByPID(int ppid, int pid)
{
  unsigned int cnt=0;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == ppid)
    {
      if(list->at(i).getPID() == pid) return cnt;
      cnt++;
    }
  }
  return cnt;
}

int ProcessList::getChildPIDByPos(int ppid, unsigned int pos)
{
  unsigned int cnt=0;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == ppid)
    {
      if(cnt == pos) return cnt;
      cnt++;
    }
  }
  return cnt;
}

unsigned int ProcessList::getChildProcessLength(int ppid)
{
  unsigned int cnt=0;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == ppid) cnt++;
  }
  return cnt;
}

Process* ProcessList::getChildProcessByPos(int ppid, unsigned int pos)
{
  unsigned int cnt=0;
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == ppid)
    {
      if(cnt==pos) return &list->at(i);
      cnt++;
    }
  }
  return getProcess(-1);
}

Process* ProcessList::getChildProcessByPID(int ppid, int pid)
{
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPPID() == ppid)
    {
      if(list->at(i).getPID()== pid) return &list->at(i);
    }
  }
  return getProcess(-1);
}

Process* ProcessList::getParentProcess(int pid)
{ return getProcess(getParentPID(pid)); }

Process* ProcessList::getParentProcess(Process *p)
{ return getProcess(p->getPPID()); }

int ProcessList::getParentPID(int pid)
{
  for(unsigned int i=0; i< list->size(); i++)
  {
    if(list->at(i).getPID() == pid) return list->at(i).getPPID();
  }
  return -1;
}

